graph-box:not(.card) {
    border: 1px solid @borders;
    border-radius: 6px;
}

list row.max {
    background: alpha(@error_color, 0.3);
}

list row.max:hover {
    background: alpha(@error_color, 0.4);
}

list row {
    font-feature-settings: "tnum";

}

button.graph-switcher {
    border-style: none;
    border-radius: 0;
}

rg-graph.big {
    background-size: 57px 57px;
    background-image: repeating-linear-gradient(0deg, @borders, @borders 1px, transparent 1px, transparent 57px);
}

rg-graph.line {
    color: rgba(74,144,217,1);
}

rg-graph.stacked {
    color: rgba(74,144,217,0.325);
}

rg-graph.line_orange {
    color: rgba(255,153,0,1);
}

rg-graph.stacked_orange {
    color: rgba(255,153,0,0.353);
}

rg-graph.line_max {
    color: rgba(238,34,34,1);
}

rg-graph.stacked_max {
    color: rgba(238,34,34,0.227);
}

StorageGraph {
    color: @view_bg_color;
}

/* Speedometer */
Speedometer {
    font-feature-settings: "tnum";
}

.speedometer-outter, .speedometer-inner, .speedometer-content-area {
    border-radius: 50%;
}

.speedometer-outter {
    background-color: @accent_bg_color;
}

.speedometer-inner {
    margin: -1px;
    background: linear-gradient(90deg, transparent 50%, @view_bg_color 50%),
                linear-gradient(90deg, @view_bg_color 50%, transparent 50%);
    transition: background 200ms ease-in-out;
}

.speedometer-content-area {
    background-color: @window_bg_color;
    margin: 5px;
}

.tag {
    border-style: none;
    border-radius: 3px;
    padding: 3px 6px;
    color: @view_bg_color;
}

.tag.tag-root {
    background-color: red;
}

.tag.tag-system {
    background-color: orange;
}

.tag.tag-user {
    background: alpha(@window_fg_color, 0.4);
}

.row_tag {
    border-radius: 100%;
}

/* Storage */
.row-tag {
    border-radius: 100%;
}

.used-tag {
    background-image:
        linear-gradient(90deg,
                        #737373 33%,
                        #ffe451 33%,
                        #ffe451 66%,
                        #ff002c 66%);
}

.available-tag { background: white; border: 1px solid @borders; }
.os-tag { background: black; }

.storage-row {
    padding: 0 5px;
    font-feature-settings: "tnum";
}

.stack-children .storage-row {
    border-bottom: 1px solid @borders;    
}

.stack-children { background: alpha(@window_fg_color, 0.05); }

box.storage {
    background-color: #ffffff;
}

row.up-folder:not(:hover) {
    background: @view_bg_color;
}

row.up-folder:not(:backdrop)  {
    border-bottom: 1px solid shade(@borders, 0.8);
}

.storage-list list {
    background: @window_bg_color;
    border-color: @borders;
    border-left: none;
    border-right: none;
    border-bottom: none;
}

.storage-list list:not(:backdrop)  {
    border-top: 1px solid shade(@borders, 0.8);
}

.storage-list list.stack-header {
    padding-bottom: 1px;
    margin: 0px;
    border:none;
}

.storage-graph-view:dir(ltr) {
    border-left: 1px solid @borders;
}

.storage-graph-view:dir(rtl) {
    border-right: 1px solid @borders;
}
